<?php
/*
Plugin Name: Custom Posts Per Page
Plugin URI: http://www.jeremyfelt.com/wordpress/plugins/custom-posts-per-page/
Description: Shows a custom set number of posts depending on the page.
Version: 0.1
Author: Jeremy Felt
Author URI: http://www.jeremyfelt.com
License: GPL2
*/

/*  Copyright 2011 Jeremy Felt (email: jeremy.felt@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

register_activation_hook( __FILE__, 'cpppc_activate' );

if ( is_admin() ){
	/*	If we're on the admin screen, we'll want to make sure that
		the appropriate settings are showing up. If we add the modified
		query here, the posts view in the admin screen changes as well. */
    add_action( 'admin_menu', 'cpppc_add_settings' );
	add_action( 'admin_init', 'cpppc_register_settings' );
}else { 
	/*	If we're on any page other than the admin screen, we'll modify the query. */
	add_filter( 'request', 'cpppc_modify_query' );
}

function cpppc_activate() {
	/*	When the plugin is first activated, we'll create the options array and
		assign a default value of 10 to each of the options. */
	$cpppc_options[ 'index_count' ] = 10;
	$cpppc_options[ 'category_count' ] = 10;
	$cpppc_options[ 'tag_count' ] = 10;
	$cpppc_options[ 'author_count' ] = 10;
	$cpppc_options[ 'archive_count' ] = 10;
	$cpppc_options[ 'search_count' ] = 10;
	$cpppc_options[ 'default_count' ] = 10;

	add_option( 'cpppc_options', $cpppc_options );
}

function cpppc_add_settings() {
	/*	Add the sub-menu item under the Settings top-level menu. */
	add_options_page( __('Posts Per Page', 'post-counts' ), __('Posts Per Page', 'post-counts'), 'manage_options', 'post-count-settings', 'cpppc_view_settings' );
	
}

function cpppc_view_settings() {
	/*	Display the main settings view for Custom Posts Per Page. */
	echo '<div class="wrap">
		<div class="icon32" id="icon-options-general"></div>
			<h2>' . __( 'Custom Posts Per Page', 'post-counts' ) . '</h2>
			<p>The settings below allow you to specify how many posts per page are displayed depending on the type of page being viewed. Different values can be set for the main index page, category pages, tag pages, author pages, archive pages and search pages. A default value can also be set for any other pages not covered by this.</p>
			<p>By enabling this plugin, the setting for "Blog pages show at most" under your Reading options in WordPress is replaced.</p>
			<form method="post" action="options.php">';
	
	settings_fields( 'cpppc_options' );
	do_settings_sections( 'cpppc' );
	
	echo '<p class="submit"><input type="submit" class="button-primary" value="';
	_e('Save Changes');
	echo '" />
			</p>
			</form>
		</div>';
	
}

function cpppc_register_settings() {
	/*	Add the settings that we're going to be using for the plugin. */
	register_setting( 'cpppc_options', 'cpppc_options', 'cpppc_options_validate' );
	add_settings_section( 'cpppc_section_main', '', 'cpppc_section_text', 'cpppc' );
	add_settings_field( 'cpppc_index_count', 'Index posts per page:', 'cpppc_index_count_text', 'cpppc', 'cpppc_section_main' );
	add_settings_field( 'cpppc_category_count', 'Category posts per page:', 'cpppc_category_count_text', 'cpppc', 'cpppc_section_main' );
	add_settings_field( 'cpppc_archive_count', 'Archive posts per page:', 'cpppc_archive_count_text', 'cpppc', 'cpppc_section_main' );
	add_settings_field( 'cpppc_tag_count', 'Tag posts per page:', 'cpppc_tag_count_text', 'cpppc', 'cpppc_section_main' );
	add_settings_field( 'cpppc_author_count', 'Author posts per page:', 'cpppc_author_count_text', 'cpppc', 'cpppc_section_main' );
	add_settings_field( 'cpppc_search_count', 'Search posts per page:', 'cpppc_search_count_text', 'cpppc', 'cpppc_section_main' );
	add_settings_field( 'cpppc_default_count', 'Default posts per page:', 'cpppc_default_count_text', 'cpppc', 'cpppc_section_main' );
}

function cpppc_options_validate( $input ) {
	/*	We aren't doing heavy validation yet, more like a passive agressive failure.
		If you enter anything other than an integer, the value will be set to 0 by 
		default and if a negative value is inputted, it will be corrected to positive. */
	$input[ "index_count" ]    = absint( intval( $input[ "index_count" ] ) );
	$input[ "category_count" ] = absint( intval( $input[ "category_count" ] ) );
	$input[ "archive_count" ]  = absint( intval( $input[ "archive_count" ] ) );
	$input[ "tag_count" ]      = absint( intval( $input[ "tag_count" ] ) );
	$input[ "author_count" ]   = absint( intval( $input[ "author_count" ] ) );
	$input[ "search_count" ]   = absint( intval( $input[ "search_count" ] ) );
	$input[ "default_count" ]  = absint( intval( $input[ "default_count" ] ) );
	
	return $input;
}

function cpppc_section_text() {
	/*	No output necessary at this time as we have only one section, though from what
		I can tell, this function is required anyway. */
}

function cpppc_index_count_text() {
	/*	Display the input field for the index page post count option. */
	$cpppc_options = get_option( 'cpppc_options' );
	
	echo '<input id="cpppc_index_count" name="cpppc_options[index_count]" size="10" type="text" value="';
	echo $cpppc_options[ 'index_count' ];
	echo '" />';
}

function cpppc_category_count_text() {
	/*	Display the input field for the category page post count option. */
	$cpppc_options = get_option( 'cpppc_options' );
	
	echo '<input id="cppppc_category_count" name="cpppc_options[category_count]" size="10" type="text" value="';
	echo $cpppc_options[ 'category_count' ];
	echo '" />';
}

function cpppc_archive_count_text() {
	/*	Display the input field for the archive page post count option. */
	$cpppc_options = get_option( 'cpppc_options' );
	
	echo '<input id="cppppc_archive_count" name="cpppc_options[archive_count]" size="10" type="text" value="';
    echo $cpppc_options[ 'archive_count' ];
	echo '" />';
}

function cpppc_tag_count_text() {
	/*	Display the input field for the tag page post count option. */
	$cpppc_options = get_option( 'cpppc_options' );
	
	echo '<input id="cpppc_tag_count" name="cpppc_options[tag_count]" size="10" type="text" value="';
	echo $cpppc_options[ 'tag_count' ];
	echo '" />';
}

function cpppc_author_count_text() {
	/*	Display the input field for the author page post count option. */
	$cpppc_options = get_option( 'cpppc_options' );
	
	echo '<input id="cpppc_author_count" name="cpppc_options[author_count]" size="10" type="text" value="';
	echo $cpppc_options[ 'author_count' ];
	echo '" />';
}

function cpppc_search_count_text() {
	/*	Display the input field for the search page post count option. */
	$cpppc_options = get_option( 'cpppc_options' );
	
	echo '<input id="cppppc_search_count" name="cpppc_options[search_count]" size="10" type="text" value="';
	echo $cpppc_options[ 'search_count' ];
    echo '" />';
}

function cpppc_default_count_text() {
	/*	Display the input field for the default page post count option. */
	$cpppc_options = get_option( 'cpppc_options' );
	
	echo '<input id="cppppc_default_count" name="cpppc_options[default_count]" size="10" type="text" value="';
	echo $cpppc_options[ 'default_count' ];
    echo '" />';
}

function cpppc_modify_query( $request ) {
	/*	This is the important part of the plugin that actually modifies the query
		at the beginning of the page before anything is displayed. */
	$cpppc_options = get_option( 'cpppc_options' );
	
	$cpppc_query = new WP_Query();
	$cpppc_query->parse_query( $request );
	
	if ( $cpppc_query->is_home() ) {
		$request[ 'posts_per_page' ] = $cpppc_options[ 'index_count' ];
	}elseif ( $cpppc_query->is_category() ) {
		$request[ 'posts_per_page' ] = $cpppc_options[ 'category_count' ];
	}elseif ( $cpppc_query->is_tag() ) {
		$request[ 'posts_per_page' ] = $cpppc_options[ 'tag_count' ];
	}elseif ( $cpppc_query->is_author() ) {
		$request[ 'posts_per_page' ] = $cpppc_options[ 'author_count' ];
	}elseif ( $cpppc_query->is_search() ) {
		$request[ 'posts_per_page' ] = $cpppc_options[ 'search_count' ];
	}elseif ( $cpppc_query->is_archive() ) {
		/*	Note that the check for is_archive needs to be below anything else
			that WordPress may consider an archive. This includes is_tag, is_category, is_author
			and probably some others.
		*/
		$request[ 'posts_per_page' ] = $cpppc_options[ 'archive_count' ];
	}else{
		$request[ 'posts_per_page' ] = $cpppc_options[ 'default_count' ];
	}

	return $request;
}
	
?>